/*
    control++ : System configuration tool
    Copyright (C) 2017-2019 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef filesystemH
#define filesystemH

#include "libcontrol++.h"
#include <vector>
#include <functional>
#include <dirent.h>

#define NOT_A_DIR   0       // The file is not a dir
#define IS_DIR      1       // The file is a dir

//-----------------------------------------------------------------------------

// Check file existence
bool CheckFile(const string& path);
// Check file extension
bool CheckExtension(const string& filename, const string& ext);
// Cut file extension
string CutExtension(const string& filename);
// Read file to standard output
bool PrintFile(const string& src_path);
// Write data string to the file from start to end index of data string
bool WriteToFile(const string& fname, const string& data, int start=-1,
        int end=-1);
// Copy file src_path to dst_path
bool CopyFile(const string& src_path, const string& dst_path);
// Format given file path
string FormatFilePath(string path, int is_dir=-1,
        bool check_special_words=true);
// Get home directory full path
string GetHomeDir(void);
// Generating unique file name
string GenFileName(const string& base, const string& postfix="");
// Strip path from filename
string StripPath(const string& path);
// End path with slash symbol
string DirEnding(const string& path);
// Given dirname is a name of the special reference dir (./ or ../)
bool DotsDir(const string& dir_name);
// Compare filenames - name colud be diranme or name of the regular file
bool CompareFilenames(const string& name1, const string& name2);
// Joing two file paths
string JoinFilePaths(const string& path_left, const string& path_right);
// Relocate given file paths to given base directory
vector<string> RebaseFilePaths(const vector<string>& paths,
        const string& base_dir);
// Checking if file is in the list of file paths or is in the dir,
// that is in the list of file paths
bool FileIsSomewhereInThere(const string& fname, const vector<string>& paths,
        bool do_format_fname=true, bool do_format_paths=true);
bool FileIsSomewhereInThere(const string& fname, const string& path,
        bool do_format_fname=true, bool do_format_paths=true);
// Going up in the file path
string Backspace(const string& path);
// Open the directory and call the function
bool OpenAndPerform(const string& dir_path,
        function<bool(const string& dp, DIR *dir, string& res)> func,
        string& result, string& error);
// Open the directory and call the function for all the files of that dir
bool OpenAndPerformOnAllFiles(const string& dir_path,
        function<bool(const string& dp, dirent *ent)> func,
        string& error);
// Check if file is a directory
bool IsDir(const string& path);

//-----------------------------------------------------------------------------

// File in memory
class TFileInMem{
    private:
    public:
        string fname;  // File full path
        string data;   // File data
        // Data character access
        char & operator[](int i){return data[i];}
        // Read file to the memory
        bool ReadFileToMem(const string& file_path);
        // Write part of the data back to the fname
        bool WriteBackToFile(int start=-1, int end=-1){
            return WriteToFile(fname, data, start, end);
            }
        // Get first character index of the n-th string after the start-index
        int NLineIndex(int n, int start=0) const;
        // Sort lines of the file in alphabetical order
        void SortLines(void);

        TFileInMem(void){}
        ~TFileInMem(void){}
    };

//-----------------------------------------------------------------------------

#endif
