/*
    control++ : System configuration tool
    Copyright (C) 2017-2019 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef libcontrolppH
#define libcontrolppH

#include <iostream>

using namespace std;

//-----------------------------------------------------------------------------

// Display error message and quit with exit code 1
void FatalError(const string& err_msg);
// Yes/No dialog
bool Approval(const string& message, const string& positive_answer);

//-----------------------------------------------------------------------------

// Adding element B to the A vector with memory reservation
template<typename A, typename B> void PushBack(A& p, const B& new_element,
        int mem_step=1, int mem_step_lim=-1){
    if((p.size() + 1) > p.capacity()){
        if(mem_step_lim > 0 && mem_step > mem_step_lim){
            mem_step = mem_step_lim;
            }
        p.reserve(p.capacity() + mem_step);
        }
    p.push_back(new_element);
    }
// Adding n-th element of source vector to dest vector
template<typename A> void PushNthBack(A& dest, const A& source, int n,
        int mem_step=1, int mem_step_lim=-1){
    if(n < (int)source.size()){
        PushBack(dest, source.at(n), mem_step, mem_step_lim);
        }
    }
// Adding all elements of source vector to dest vector
template<typename A> void PushAllBack(A& dest, const A& source,
        int mem_step=-1, int mem_step_lim=-1){
    if(mem_step < 0){
        mem_step = source.size();
        }
    for(auto & src : source){
        PushBack(dest, src, mem_step, mem_step_lim);
        }
    }

//-----------------------------------------------------------------------------

#endif
