/*
    control++ : System configuration tool
    Copyright (C) 2017-2019 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef permissions_basicH
#define permissions_basicH

#include "libcontrol++.h"

#include <unistd.h>         // Contains "chown" function description
#include <pwd.h>            // Contains "passwd" struct description
#include <grp.h>            // Contains "group" struct description
#include <sys/acl.h>        // Contains "acl_*" functions
#include <acl/libacl.h>
#include <map>

#define ALLOW_EXE       "**x**x**x" // Mode string to allow execution
#define DISALLOW_EXE    "**-**-**-" // Mode string to disallow execution
#define DONT_CHANGE     "*********" // Unvarying mode string
#define DISALLOW_ALL    "---------" // Unvarying mode string

//-----------------------------------------------------------------------------

// rwx - 'r' (read), 'w' (write) or 'x' (execute) mode
extern array<char, 3> rwx;
// uqo - 'u' (user), 'g' (group) or 'o' (others) attachment
extern array<char, 3> ugo;
// Read, write or execute ACL permission types
extern array<acl_perm_t, 3> acl_perm_types;

//-----------------------------------------------------------------------------

// Get id of the file's owner
bool GetUserId(uid_t& uid, const string& user, string& error);
// Get id of the file's group
bool GetGroupId(gid_t& gid, const string& group, string& error);

// Get name of the file's owner
string GetUserName(uid_t uid, string& error);
// Get name of the file's group
string GetGroupName(gid_t gid, string& error);
// Verify owner of the file, presented by stat
bool VerifyOwner(const struct stat &st, const string &owner);
// Verify group of the file, presented by stat
bool VerifyGroup(const struct stat &st, const string &group);
// Set according bit of the mode
void SetPermModeBit(struct stat &st, char new_state, char rwx, char ugo);
// Get bit mask with single 1 bit for the given rwx & ugo
mode_t GetPermModeMask(char rwx, char ugo, bool with_spec_bit=false);
// Check according bits of the mode field of the particular file stat structure
bool CheckPermModeBit(const struct stat &st, char b, char rwx, char ugo);
// Compare permission modes
bool ComparePermModes(mode_t mode_a, const string &mode_b);
bool ComparePermModes(const string &mode_a, const string &mode_b);
// Get ls -l type of mode string from the t_mode
string GetPermModeString(mode_t mode);
// Type of exe permission (with/without suid/sticky bit)
char XPerm(mode_t mode, mode_t exe_bit, mode_t spec_bit);
// Permission bit is read, wright or exe bit
bool IsRWX(char b);
// Check format of the mode string
bool CheckModeString(const string &mode);
// Check format of the section (rwx) of the mode string
bool CheckModeSection(string::const_iterator mode_sec, char ugo='\0');

// Update ACL permission string
bool UpdateACLPermStr(string &perm, acl_permset_t permset, acl_perm_t type,
        string& error);

//-----------------------------------------------------------------------------

#endif
