/*
    control++ : System configuration tool
    Copyright (C) 2017-2019 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef textH
#define textH

#include "libcontrol++.h"
#include <array>

extern array<char, 4> esc_symbols;

//-----------------------------------------------------------------------------

// Conver string to uppercase
string ToUpper(string str);
// Conver string to lowercase
string ToLower(string str);
// Position of n-th ocurrance of a substring in a string
int FindSubstrNthOccurrence(int n, const string& subs, const string& str);
// Mirror single quotes
void MirrorSingleQuotes(string& line);
// Search for the next word in the string and return the index
// of the last character of the word
int GetWordFromLine(const string& line, string& word, int start=-1, int end=-1);
// Trim chars from the left
void LTrim(string &s, const string &chars="");
// Trim chars from the right
void RTrim(string &s, const string &chars="");
// Trim chars from both sides
void Trim(string &s, const string &chars="", bool ltrim=true, bool rtrim=true);
// Cut comment from given line
void CutComment(string& line, const string& comment="#");
// Escape special symbols
string Esc(const string &str);
// Unescape special symbols
string Unesc(const string &str);
// Check if symbol is escaped
bool Escaped(const string &str, int index);
// Find position of the first unescaped quotation mark in the string
// starting from the pos
int UnescapedQtnMarkPos(const string &str, int pos, bool double_quote);

//-----------------------------------------------------------------------------

// String with case insensitive comparisment
struct ci_char_traits : public char_traits<char>{
    static bool eq(char c1, char c2){return toupper(c1) == toupper(c2);}
    static bool ne(char c1, char c2){return toupper(c1) != toupper(c2);}
    static bool lt(char c1, char c2){return toupper(c1) < toupper(c2);}
    static int compare(const char* s1, const char* s2, size_t n){
        while(n != 0){
            if(toupper(*s1) < toupper(*s2)){
                return -1;
                }
            if(toupper(*s1) > toupper(*s2)){
                return 1;
                }
            n--;
            s1++;
            s2++;
            }
        return 0;
        }
    static const char* find(const char* s, int n, char a){
        while(n>0 && toupper(*s) != toupper(a)){
            n--;
            ++s;
            }
        return s;
        }
    };
typedef basic_string<char, ci_char_traits> ci_string;

//-----------------------------------------------------------------------------

#endif
