/*
    control++ : System configuration tool
    Copyright (C) 2017-2019 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef vectorH
#define vectorH

#include "libcontrol++.h"
#include <vector>
#include <set>

//-----------------------------------------------------------------------------

// Remove empty strings from the vector
void VRemEmpty(vector<string>& v);

//-----------------------------------------------------------------------------

// Get index of the vector element
template<typename A> int VIndex(const vector<A>& v, const A& el){
    if(v.size() <= 0){
        return -1;
        }
    for(int i = 0; i < (int)v.size(); i++){
        if(el == v[i]){
            return i;
            }
        }
    return -1;
    }

//-----------------------------------------------------------------------------

// Check if vector contents given value
template<typename A> bool VContents(const vector<A>& v, const A& el){
    return ((VIndex(v, el) >= 0) ? true : false);
    }

//-----------------------------------------------------------------------------

// Get subvector of the given vector
template<typename A> vector<A> VSub(const vector<A>& v, int start,
        int length=-1){
    auto first = (start >= 0 && start <= (int)v.size()) ?
        (v.begin() + start) : v.begin();
    auto last = (length >= 0 && length <= (v.end() - first)) ?
        (first + length) : v.end();
    return vector<A> (first, last);
    }

//-----------------------------------------------------------------------------

// Get elements of vector x that are presented in vector y
template<typename A> vector<A> VIntersec(const vector<A> &x, const vector<A> &y){
    vector<A> intersec_vec;
    set<A> intersec_set;
    set<A> y_set(y.begin(), y.end());
    for(auto & x_el : x){
        if(y_set.find(x_el) != y_set.end() &&
                intersec_set.find(x_el) != intersec_set.end()){
            intersec_vec.push_back(x_el);
            intersec_set.add(x_el);
            }
        }
    return intersec_vec;
    }

//-----------------------------------------------------------------------------

// Get elements of vector x that are not presented in vector y
template<typename A> vector<A> VMinus(const vector<A> &x, const vector<A> &y){
    vector<A> diff;
    set<A> y_set(y.begin(), y.end());
    for(auto & x_el : x){
        if(y_set.find(x_el) == y_set.end()){
            diff.push_back(x_el);
            }
        }
    return diff;
    }

//-----------------------------------------------------------------------------

#endif
